VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Algorithms"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

' Searching and sorting algorithms for zero-based
' variant two dimensional arrays where the first
' dimension is the columns and the second is the rows

Public Sub BubbleSort(ByRef varArray As Variant, _
    Optional flDescending As Boolean, Optional ByVal pIndex As Integer)

Dim i As Long
Dim j As Long
Dim lngUB As Long
Dim lngLB As Long
Dim lngUB1 As Long
Dim lngLB1 As Long
Dim z As Long
Dim varArrTemp As Variant

' Cache the bounds
lngUB = UBound(varArray, 2)
lngLB = LBound(varArray, 2)

lngUB1 = UBound(varArray, 1)
lngLB1 = LBound(varArray, 1)

' If the optional index is 0 then set it to the
' lower bound (sort by first column)
If pIndex = 0 Then
    pIndex = LBound(varArray, 1)
End If

' Loop through the array using the second
' dimension of the array (the rows)
For i = lngLB To lngUB
    For j = lngLB To lngUB - 1 - i
        ' Compare the items in the array
        If CompMe(varArray(pIndex, j), varArray(pIndex, j + 1), flDescending) Then
            ReDim varArrTemp(UBound(varArray, 1), 0)
            ' If the first item is larger then swap them
            For z = lngLB1 To lngUB1
                varArrTemp(z, 0) = varArray(z, j)
            Next z
            For z = lngLB1 To lngUB1
                varArray(z, j) = varArray(z, j + 1)
            Next z
            For z = lngLB1 To lngUB1
                varArray(z, j + 1) = varArrTemp(z, 0)
            Next z
        End If
    Next j
Next i

End Sub


Public Sub ShellSort(varArray As Variant, _
    Optional flDescending As Boolean, Optional ByVal pIndex As Integer)
    
Dim i As Long
Dim lngPos As Long
Dim varTemp As Variant
Dim lngLB As Long
Dim lngUB As Long
Dim lngSkip As Long
Dim flDone As Boolean
Dim z As Long
Dim varArrTemp As Variant


' If the optional index is 0 then set it to the
' lower bound (sort by first column)
If pIndex = 0 Then
    pIndex = LBound(varArray, 1)
End If

' Cache the lower and upper bounds
lngLB = LBound(varArray, 2)
lngUB = UBound(varArray, 2)

' Assign the skip count
Do
    lngSkip = (3 * lngSkip) + 1
Loop Until lngSkip > lngUB

Do
    ' Decrement the skip each time through the loop
    lngSkip = lngSkip / 3
    
    ' Check the remainder of the array
    For i = lngSkip + 1 To lngUB
        ' Pick up the current value
        varTemp = varArray(pIndex, i)
        ReDim varArrTemp(UBound(varArray, 1), 0)
        For z = LBound(varArray, 1) To UBound(varArray, 1)
            varArrTemp(z, 0) = varArray(z, i)
        Next z
        lngPos = i
        
        ' If we've reached the beginning then increment the
        ' skip count but signal that this is the last pass
        If lngSkip = 0 Then
            lngSkip = 1
            flDone = True
        End If
        ' Check to see if the preceding element is larger
        Do While CompMe(varArray(pIndex, lngPos - lngSkip), varTemp, flDescending)
            ' If so then slide it in
            For z = LBound(varArray, 1) To UBound(varArray, 1)
                varArray(z, lngPos) = varArray(z, lngPos - lngSkip)
            Next z
            lngPos = lngPos - lngSkip
            If lngPos <= lngSkip Then Exit Do
        Loop
        ' Put the current value back down
        For z = 0 To UBound(varArray, 1)
            varArray(z, lngPos) = varArrTemp(z, 0)
        Next z
    Next i
        
Loop Until lngSkip = lngLB Or flDone

End Sub



Private Function CompMe(ByVal pArg1 As Variant, ByVal pArg2 As Variant, _
    ByVal pDesc As Boolean, Optional pEqual As Boolean) As Boolean

CompMe = False

' If descending then do a less than compare
If pDesc Then
    Select Case pEqual
    ' If equality is specified then use an equal sign
    Case True
        ' Check if its a string to do a string compare
        If VarType(pArg1) = vbString Then
            If UCase(pArg1) <= UCase(pArg2) Then
                CompMe = True
            End If
        Else
            If pArg1 <= pArg2 Then
                CompMe = True
            End If
        End If
    Case False
        ' If not specified then do a < compare
        If VarType(pArg1) = vbString Then
            If StrComp(pArg1, pArg2, vbTextCompare) = -1 Then
                CompMe = True
            End If
        Else
            If pArg1 < pArg2 Then
                CompMe = True
            End If
        End If
    End Select
Else
    ' If ascending doing a greater than compare
    Select Case pEqual
    Case True
        ' Check if its a string first
        If VarType(pArg1) = vbString Then
            If UCase(pArg1) >= UCase(pArg2) Then
                CompMe = True
            End If
        Else
            If pArg1 >= pArg2 Then
                CompMe = True
            End If
        End If
    Case False
        ' Check if its a string
        If VarType(pArg1) = vbString Then
            If StrComp(pArg1, pArg2, vbTextCompare) = 1 Then
                CompMe = True
            End If
        Else
            If pArg1 > pArg2 Then
                CompMe = True
            End If
        End If
    End Select
End If

End Function


Public Function BinSearch(varArray As Variant, _
    ByVal varSearch As Variant, ByVal pIndex As Integer, _
    Optional ByVal flPartial As Boolean) As Long
  
Dim lngLow As Long
Dim lngUpper As Long
Dim lngPos As Long

' Set the upper and lower bounds of the array
lngLow = LBound(varArray, 2)
lngUpper = UBound(varArray, 2)

Do While True
    ' Divide the array to search
    lngPos = (lngLow + lngUpper) / 2
    
    ' Look for a match
    If flPartial Then
        ' If partial is specified then do a comparison
        ' on the substring since it should be a string
        If StrComp(Mid(varArray(pIndex, lngPos), 1, Len(varSearch)), _
                varSearch, vbTextCompare) = 0 Then
            ' If we've found it then get out
            BinSearch = lngPos
            Exit Function
        End If
    Else
        ' Check to see if its a string
        If VarType(varSearch) = vbString Then
            If StrComp(varArray(pIndex, lngPos), varSearch, _
                    vbTextCompare) = 0 Then
                ' If we've found it then get out
                BinSearch = lngPos
                Exit Function
            End If
        Else
            If varArray(pIndex, lngPos) = varSearch Then
                ' If we've found it then get out
                BinSearch = lngPos
                Exit Function
            End If
        End If
    End If
        ' Check to see if its the last value to be checked
        If lngUpper = lngLow + 1 Then
            lngLow = lngUpper
        Else
            ' If we get to the lowest position then it's not there
            If lngPos = lngLow Then
                BinSearch = -1
                Exit Function
            Else
                ' Determine whether to look in the upper or
                ' lower half of the array
                If VarType(varSearch) = vbString Then
                    If StrComp(varArray(pIndex, lngPos), _
                            varSearch, vbTextCompare) = 1 Then
                        lngUpper = lngPos
                    Else
                        lngLow = lngPos
                    End If
                Else
                    If varArray(pIndex, lngPos) > varSearch Then
                        lngUpper = lngPos
                    Else
                        lngLow = lngPos
                    End If
                End If
            End If
        End If
Loop

End Function







