Option Strict On

Imports System.IO
Imports System.ComponentModel

Namespace Quilogy.Utils

  <DefaultProperty("Source")> _
  Public Class QuilogyLog : Inherits Component

    ' Logging class
    Private mstrSource As String
    Private mstrFile As String
    Private mflFileOnly As Boolean

    Public Sub New()
      ' Default the value
      Me.LogFile = "AppLog.txt"
      Me.FileOnly = False
    End Sub

    Public Sub New(ByVal source As String, ByVal fileName As String, _
      ByVal fileOnly As Boolean)
      Me.Source = Source
      Me.LogFile = fileName
      Me.FileOnly = FileOnly
    End Sub

    <Browsable(True), _
    Category("Logging Options"), _
    Description("Path of the file used to log to.")> _
    Public Property LogFile() As String
      Get
        Return mstrFile
      End Get
      Set(ByVal Value As String)
        mstrFile = Value
      End Set
    End Property

    <Browsable(True), _
    Category("Logging Options"), _
    Description("Log source to use in the Application log")> _
    Public Property Source() As String
      Get
        Return mstrSource
      End Get
      Set(ByVal Value As String)
        mstrSource = Value
      End Set
    End Property

    <Browsable(True), _
    Category("Logging Options"), _
    Description("Log to the text file only?")> _
    Public Property FileOnly() As Boolean
      Get
        Return mflFileOnly
      End Get
      Set(ByVal Value As Boolean)
        mflFileOnly = Value
      End Set
    End Property

    Public Sub LogText(ByVal message As String, _
      ByVal isError As Boolean)
      ' Write to both the text file and the Quilogy log
      Dim objLog As New EventLog("Application")
      Dim strType As String

      objLog.Source = mstrSource

      If Len(message) = 0 Then
        Throw New ArgumentException("No message passed.")
        Return
      End If

      Try
        ' Create or open the log file
        If Len(mstrFile) = 0 Then
          Throw New Exception("Must set the LogFile property first")
          Return
        End If

        Dim objFs As FileStream = New FileStream(mstrFile, _
          FileMode.OpenOrCreate, FileAccess.Write)
        Dim objSw As StreamWriter = New StreamWriter(objFs)
        objSw.BaseStream.Seek(0, SeekOrigin.End) ' point to the end

        ' Write the log entry
        objSw.Write("Log Entry : ")
        objSw.Write("{0} {1}", DateTime.Now.ToShortTimeString(), _
           DateTime.Now.ToShortDateString())

        If isError Then
          strType = "Error"
        Else
          strType = "Information"
        End If

        objSw.WriteLine(":" & strType & "[" & message & "]")
        objSw.Flush()  ' Update underlying file
        objSw.Close()

      Catch e As Exception
        Throw New Exception("Could not create or open " & _
          mstrFile & ": " & e.Message)

      Finally
        If Not mflFileOnly Then
          Try
            ' Make sure the source is registered, it should be
            If (Not EventLog.SourceExists(objLog.Source)) Then
              EventLog.CreateEventSource(objLog.Source, "Application")
            End If

            ' Write the message to the log
            If isError Then
              objLog.WriteEntry(message, EventLogEntryType.Error)
            Else
              objLog.WriteEntry(message, EventLogEntryType.Information)
            End If
          Catch e As Exception
            ' Not so important so skip it
            Throw New Exception("Error logging to Application log: " & _
              e.Message)
          Finally
            objLog.Dispose()
          End Try
        End If
      End Try

    End Sub

  End Class

End Namespace
