/*+==========================================================================
  File:      ftpreq.cool

  Summary:   This file implements the ftp protocol using the Pluggable protocol feature 
             of net classes, he purpose of this to serve as the sample for pluggable protocol
			 and test the pluggable protocol features.  

  Classes:   FtpWebRequest
==========================================================================+*/

using System;
using System.IO;
using System.Text;
using System.Net;
using System.Net.Sockets;

public  enum  FtpCommandType
{
	FtpControlCommand	   = 1,
	FtpDataRecieveCommand  = 2,
	FtpDataSendCommand	   = 3,
	FtpCommandNotSupported = 4,
}

public class FtpRequestCreator : IWebRequestCreate
{
    public FtpRequestCreator()
    {
    }
    
    public WebRequest Create(Uri  Url)

    {
        return new FtpWebRequest(Url);
    }
    
};

public class FtpWebResponse : WebResponse
{
	// Properties overriding webresponse
	public FtpWebResponse()
	{
	}
	private int _StatusCode;
	private String _StatusDescription;
	private String _ContentEncoding ;
	private Stream _ResponseStream ;	
	
	public String StatusDescription
	{
		get { return _StatusDescription;}
		 set { _StatusDescription=value; }		

	}	
	public String ContentEncoding 
	{
		get	{ return _ContentEncoding; } 
		set	{ _ContentEncoding = value;}
	}
	public override Stream GetResponseStream()
	{
		return _ResponseStream;	
	}	
	public int Status
	{
		 get { return _StatusCode;}				
		 set { _StatusCode=value;}				
	}	
	
	public void StoreData(Stream datastream)
	{
		_ResponseStream = datastream;		
	}
	 
}
public class FtpWebRequest : WebRequest
{
    private const int SOCKET_ERROR = -1;

    public String CmdParameter;	
    private Socket DataSocket;
    private Socket ControlSocket;		
	
    private string _Method;	
    private Uri _Url;
    private MemoryStream _DataStreamToUpload;

    private bool _bPassiveMode;

    public void SetPassiveMode()
    {
        _bPassiveMode = true;
    }

    override public String Method
    {	
        get 
        {
            return _Method;
        }
        set 
        {
            _Method=value;
        }
    }
    public override Stream GetRequestStream()
    {	
        if(	_DataStreamToUpload == null)
            _DataStreamToUpload = new MemoryStream();	
        return _DataStreamToUpload;		
    }
	
    public FtpWebRequest(Uri Url)
    {
        // do whatever initialization is required here
        if(Url.Scheme != "ftp")  // This class is only for ftp urls
            throw new NotSupportedException();
        _Url = Url;
        _Method = "RETR";  // default is to retrieve a file
        _bPassiveMode = false;
        CmdParameter = Url.AbsolutePath; // default is the path of url is path of file to retrieve
        OpenControlConnection();
    }

	public override WebResponse  GetResponse()
	{
		FtpCommandType commandtype = FindCommandType(_Method);			
		FtpWebResponse ftpresponse = new FtpWebResponse();			
		if(commandtype == FtpCommandType.FtpDataRecieveCommand 
			|| commandtype == FtpCommandType.FtpDataSendCommand)
		{
			if(_bPassiveMode)
			{
				OpenPassiveDataConnection();								
			}
			else
				OpenDataConnection();									
		}
		SendCommand(Method, CmdParameter);
		ftpresponse = ReceiveResponse();			
		if(commandtype == FtpCommandType.FtpDataSendCommand)
		{				
			if(ftpresponse.Status/100 == 1) // Positive preliminary reply
			{
				if(_DataStreamToUpload != null)
				{
					Socket DataConnection;
					if( _bPassiveMode)
						DataConnection = DataSocket;							
					else
						DataConnection = DataSocket.Accept();								
					if(DataConnection == null)
					{
						throw new ProtocolViolationException("Accept failed ");						
					}
					SendData(DataConnection);							
					DataConnection.Close();										
					ftpresponse = ReceiveResponse();
				}
				else
				{  // Data to be send is not specified
					throw new ProtocolViolationException();
				}					
			}
			else
			{
				Console.WriteLine(ftpresponse.StatusDescription);	
			}
			CloseDataConnection();
		}
		else if(commandtype == FtpCommandType.FtpDataRecieveCommand)
		{					
			if(ftpresponse.Status/100 == 1) // Positive preliminary reply
			{
				Socket DataConnection;
				if( _bPassiveMode)
					DataConnection = DataSocket;							
				else
					DataConnection = DataSocket.Accept();											
				if(DataConnection == null)
				{
					throw new ProtocolViolationException("DataConnection failed ");
				}					
				Stream datastream = ReceiveData(DataConnection);								
				ftpresponse = ReceiveResponse();					
				ftpresponse.StoreData(datastream);
			}
			CloseDataConnection();
		}							
		return ftpresponse;
	}	

	private void OpenDataConnection()
	{
		if( DataSocket != null) 
		{ // Through some exception here because passed socket is already in use
			throw new ProtocolViolationException();
        }
		//was AddressFamily.AfINet
        DataSocket = new Socket( AddressFamily.InterNetwork , SocketType.Stream , ProtocolType.Tcp  );

        IPHostEntry localHostEntry = Dns.GetHostByName(Dns.GetHostName());
//		EndPoint epListener = new IPEndPoint(localHostEntry.AddressList[0], IPAddress.Any );				        
		EndPoint epListener = new IPEndPoint(localHostEntry.AddressList[0],0);				        
		DataSocket.Bind(epListener);        
        DataSocket.Listen(5); // Put the dataSocket * & in Listen mode
		Console.WriteLine(DataSocket.Connected);		
		IPEndPoint localEP = (IPEndPoint) DataSocket.LocalEndPoint ;
		UInt32 localEPAddress =(UInt32) localEP.Address.Address;
        String szLocal = FormatAddress(localEPAddress, localEP.Port);						
        SendCommand("PORT",szLocal);                
        
        FtpWebResponse response = ReceiveResponse();
		Console.WriteLine(response.StatusDescription + szLocal);
        if( (response.Status/100) != 2 )
		{
            // throw some exception here
		}
	}

	private void OpenPassiveDataConnection()
	{
		if( DataSocket != null) 
		{ // Through some exception here because passed socket is already in use
			throw new ProtocolViolationException();
        }
		String IPAddressStr =null;
		int Port = 0 ;
        int cbReturn = SendCommand("PASV","");                
        if( cbReturn == SOCKET_ERROR)
        {
			// throw some exception here
		}        

		FtpWebResponse response = ReceiveResponse();
        if( (response.Status/100) != 2 )
		{
            // throw some exception here
		}
		Console.WriteLine(response.StatusDescription);
		//Find the IPAddress and port address from response
		IPAddressStr = getIPAddress(response.StatusDescription);
		Port = getPort(response.StatusDescription);
		//was AddressFamily.AfINet
        DataSocket = new Socket( AddressFamily.InterNetwork , SocketType.Stream, ProtocolType.Tcp );       		
        if( cbReturn == SOCKET_ERROR )
		{
			throw new ProtocolViolationException("Error in creating Data Socket");	
		} 					

		//IPHostEntry serverHostEntry = .GetHostByName(_Url.Host);		
		
		//IPEndPoint	serverEndPoint = new IPEndPoint(serverHostEntry.AddressList[0], Port); 		
		EndPoint	serverEndPoint = new IPEndPoint(new IPAddress(Int32.Parse(IPAddressStr)), Port); 	
		Console.WriteLine(serverEndPoint);
		try {
			DataSocket.Connect(serverEndPoint);
		}
		catch(Exception)	{
			DataSocket.Close();
			DataSocket = null;				
			throw new ProtocolViolationException("Passive connection failure");   
		}  							
		return;
	}


	private void OpenControlConnection()
	{
		String Host = _Url.Host;

		if(ControlSocket != null) // socketalready in use
		{
			throw new ProtocolViolationException();            
		}
										//was AfINet
        ControlSocket = new Socket( AddressFamily.InterNetwork , SocketType.Stream, ProtocolType.Tcp );
		//EndPoint clientIPEndPoint = new IPEndPoint(IPAddress.Any , IPAddress.Any );
		EndPoint clientIPEndPoint = new IPEndPoint(IPAddress.Any, 0);
		EndPoint clientEndPoint = clientIPEndPoint;
		try {
			ControlSocket.Bind(clientEndPoint);
		}
		catch(Exception) {
            ControlSocket.Close();
            ControlSocket = null;			
			throw new ProtocolViolationException(" Error in opening control connection");            
        }
        clientEndPoint = ControlSocket.LocalEndPoint ;
		try
		{
			IPHostEntry serverHostEntry = Dns.GetHostByName(Host);
			IPEndPoint serverEndPoint = new IPEndPoint( serverHostEntry.AddressList[0], 21 );
			try{
				ControlSocket.Connect(serverEndPoint);
			}
			catch(Exception){
	            ControlSocket.Close();
		        ControlSocket = null;
				throw new ProtocolViolationException("Error in creating control connection");
			}  	
		}
		catch ( Exception e)
		{
			ControlSocket.Close();
            ControlSocket = null;
			throw e;        
		}
        
        
		// Get the initial response after connection		
		
		MemoryStream responseStream = new MemoryStream();
		while(true)
		{			
			int BufferSize = 256;
			Byte[] recvbuffer = new Byte[BufferSize + 1];			
			int bytesread = ControlSocket.Receive(recvbuffer, BufferSize, 0 );			
			responseStream.Write(recvbuffer,0,bytesread);			
			if(IsCompleteResponse(responseStream))			
				break;			
		}
		return;		
	}

	public void CloseDataConnection()
	{
		if(DataSocket != null)
		{
			DataSocket.Close();
			DataSocket = null;		
		}
	}

	private void CloseControlConnection()
	{
		ControlSocket.Close();
		ControlSocket = null;
	}	

	private Stream ReceiveData(Socket Accept)
	{
        if( DataSocket == null ) // something went wrong, this should never happen
		{
			throw new ArgumentNullException();
        }		
		MemoryStream responseStream = new MemoryStream();
        int BufferSize = 256;
        Byte [] recvbuffer = new Byte[BufferSize + 1];        
        while(true) 
		{
            int bytesread = 0;
            recvbuffer[bytesread] = (Byte)'\0';            			          
			bytesread = Accept.Receive(recvbuffer, BufferSize, 0 );			
			if( bytesread <= 0 ) 
				break;  			
			responseStream.Write(recvbuffer,0,bytesread);														
        } 				
		//Console.WriteLine("Bytes Received " + responseStream.Length);
		return responseStream;
	}

	private FtpWebResponse ReceiveResponse()
	{ 
		bool bCompleteResponse=false;     
	    if( ControlSocket == null ) // something went wrong protocol violation exception
		{   			
            throw new ArgumentNullException();
        }
		FtpWebResponse thisftpResponse = new FtpWebResponse();
		MemoryStream responseStream = new MemoryStream();				
        while(true) 
		{			
			int BufferSize = 256;
			Byte[] recvbuffer = new Byte[BufferSize + 1];
			int bytesread = 0;
            recvbuffer[0] = (Byte)'\0';				
            bytesread = ControlSocket.Receive(recvbuffer, BufferSize, 0 );									
            if( bytesread <= 0 ) 
				break;  // No response recieve to process, exit the loop
			//copy the recieved data in responsestream					
			responseStream.Write(recvbuffer,0,bytesread);								
			bCompleteResponse = IsCompleteResponse(responseStream);				
			if(bCompleteResponse)
			{				
				break;			
			}
		}								
		if(bCompleteResponse)
		{
		// now get status code	
			try
			{
				responseStream.Position=0;
				Byte [] StatusCode = new Byte[3];
				responseStream.Read (StatusCode, 0, 3) ;
				String statuscodestr=Encoding.ASCII.GetString(StatusCode,0,3);
				thisftpResponse.Status = Int16.Parse( statuscodestr);
			}
			catch
			{
				thisftpResponse.Status =  -1;
			}					
		// Copy the response in Status Description												
			int responsesize = (int)responseStream.Length;
			responseStream.Position = 0;
			Byte [] StatusDescription = new Byte[responsesize];
			responseStream.Read (StatusDescription, 0, responsesize) ;			
			thisftpResponse.StatusDescription = Encoding.ASCII.GetString(StatusDescription,4,responsesize-4);			
		}
		else
		{
		// something went wrong here			
			throw new ProtocolViolationException ();			
		}
		return thisftpResponse;
	}
	
	private bool IsCompleteResponse(Stream responseStream)
	{		
		bool bIsComplete = false;		
		int responselength = (int) responseStream.Length;
		responseStream.Position = 0;		
		if(responselength >= 5) // statuscode(3 char)+CRLF(2 char)
		{
			int StatusCode=-1;									
			Byte[] ByteArray = new Byte[responselength];			
			String statuscodestr;							
			responseStream.Read(ByteArray,0,responselength);		
			statuscodestr=Encoding.ASCII.GetString(ByteArray,0,responselength);			
			if (responselength==5 && ByteArray[responselength-1] == '\n')	
			{
				// Last character LF seems to be very special case, need to verify					
					bIsComplete = true;
			}			
			else if ((ByteArray [responselength-1] == '\n') && (ByteArray [responselength-2] == '\r'))	
			{							
					bIsComplete = true;
			}			
			if (responselength==5 && ByteArray[responselength-1] == '\n')	
			{
				// Last character LF seems to be very special case, need to verify
					bIsComplete = true;
			}			
			if(bIsComplete) //do additional verifications
			{
				try
				{
					StatusCode = Int16.Parse( statuscodestr.Substring(0,3));
				}
				catch
				{
					StatusCode =  -1;					
				}		
				if (statuscodestr[3] == '-') 
				{
					// multiline response verify whether response is complete, reponse must be ended with CRLF					
					//find out the beginning of last line					
					int lastlinestart =0;
					for(lastlinestart=responselength-2;lastlinestart >0;lastlinestart--)
					{						
						if ( ByteArray [lastlinestart] == '\n' && ByteArray [lastlinestart-1] == '\r')
							break;						
					}					
					if(lastlinestart ==0) 
					{
							bIsComplete = false; // Multilines not recieved						
					}
					else if(statuscodestr[lastlinestart+4] != ' ') //still not completed
					{
						bIsComplete = false;
					}
					else
					{
						int endcode = -1;						
						try
						{
							endcode = Int16.Parse( statuscodestr.Substring(lastlinestart+1,3));
						}
						catch
						{
							endcode = -1;
						}		
						if (endcode != StatusCode)
								bIsComplete = false; // error invalid response					
					}
						
				}
				else if (statuscodestr[3] != ' ')
				{
					StatusCode = -1;														
				}
			}
		}
		else
		{			
			bIsComplete = false;						
		}
		return bIsComplete;                
	}
	
	// Following function will return the status code if response is OK else return -1

	public static FtpCommandType FindCommandType(String command)
	{	
		// Currently supprting only these four commands which return data on datasocket
		if(command.Equals("USER") || command.Equals("PASS") || command.Equals("CWD") 
			|| command.Equals("PWD")|| command.Equals("CDUP") || command.Equals("QUIT"))
			return FtpCommandType.FtpControlCommand;
		else if(command.Equals("RETR")||command.Equals("LIST"))
			return FtpCommandType.FtpDataRecieveCommand;
		else if(command.Equals("STOR")|| command.Equals("STOU"))
			return FtpCommandType.FtpDataSendCommand;
		else 
			return FtpCommandType.FtpCommandNotSupported;
	}

	private int SendCommand (String RequestedMethod, String Parametertopass)
	{
	    String Command = RequestedMethod;
		if(Parametertopass != null && !Parametertopass.Equals(""))
			Command = Command +	" " + Parametertopass ;
		Command = Command +  "\r\n";
		//Console.WriteLine(Command);
	    Byte[] sendbuffer = Encoding.ASCII.GetBytes(Command.ToCharArray());		
		if( ControlSocket == null ) 
		{
			throw new ProtocolViolationException ();			
		}
		int cbReturn = ControlSocket.Send( sendbuffer,Command.Length, 0);
		return (cbReturn);
	}
		
	private int SendData(Socket Accept)
	{
		if( Accept == null ) 
		{
			throw new ArgumentNullException();			
		}
		int Length = (int) _DataStreamToUpload.Length;
		_DataStreamToUpload.Position=0;
		Byte [] sendbuffer = new Byte[Length];

		_DataStreamToUpload.Read(sendbuffer,0,Length);		
		int cbReturn = Accept.Send( sendbuffer, Length, 0);
		return cbReturn;
	}
	private String FormatAddress(UInt32 Address, int Port )
	{
		StringBuilder sb = new StringBuilder(32);
	
		UInt32 Quotient = Address;
		UInt32 Remainder;	
		while( Quotient!=0 ) 
		{
			Remainder = Quotient % 256;
			Quotient = Quotient / 256;
			sb.Append(Remainder);
			sb.Append(',');
		}
		sb.Append( Port / 256 );
		sb.Append(',');
		sb.Append(Port % 256 );
		return sb.ToString();
	}
	private String getIPAddress(String str)
	{
		StringBuilder IPstr=new StringBuilder(32);
		String Substr = null;
		int pos1 = str.IndexOf("(")+1;
		int pos2 = str.IndexOf(",");
		for(int i =0; i<3;i++)
		{
			Substr = str.Substring(pos1,pos2-pos1)+".";
			IPstr.Append(Substr);
			pos1 = pos2+1;
			pos2 = str.IndexOf(",",pos1);
		}		
		Substr = str.Substring(pos1,pos2-pos1);
		IPstr.Append(Substr);		
		return IPstr.ToString();
	}
	private int getPort(String str)
	{
		int Port=0;
		int pos1 = str.IndexOf("(");
		int pos2 = str.IndexOf(",");
		for(int i =0; i<3;i++) //skip the ip addresss
		{			
			pos1 = pos2+1;
			pos2 = str.IndexOf(",",pos1);
		}		
		pos1 = pos2+1;
		pos2 = str.IndexOf(",",pos1);		
		String PortSubstr1=str.Substring(pos1,pos2-pos1);
		
		pos1 = pos2+1;
		pos2 = str.IndexOf(")",pos1);
		String PortSubstr2=str.Substring(pos1,pos2-pos1);
		
		//evaluate port number
		Port = Int32.Parse(PortSubstr1)*256 ;
		Port = Port + Int32.Parse(PortSubstr2);									
		return Port;
	}

}