Option Strict Off

Imports System.ComponentModel
Imports System.ServiceProcess
Imports System.Collections
Imports System.Configuration.Install
Imports Microsoft.Win32
Imports System.Timers
Imports System.IO
Imports System.Diagnostics
Imports System.Net
Imports System.Messaging


Module Services
    Public Const QUILOGY_APPLOG_SOURCE As String = "QXMLTransfer"
    Public Const QUILOGY_CUSTLOG_SOURCE As String = "QuilogyXMLTransfer"

    ' The main entry point for the process
  Public Sub Main()

    Dim ServicesToRun() As ServiceBase

    ' More than one NT Service may run within the same process. To add
    ' another service to this process, change the following line to
    ' create a second service object. For example,
    '
    'ServicesToRun = New System.ServiceProcess.ServiceBase() {New QXMLTransfer(), New serv2()}
    '
    ' Set the properties
    Dim svcXML As New QXMLTransfer()

    'ServicesToRun = New System.ServiceProcess.ServiceBase() {svcXML}

    'could also use the alternative constructor
    'System.ServiceProcess.ServiceBase.Run(ServicesToRun)
    ServiceBase.Run(svcXML)

  End Sub

  Friend Sub WriteLog(ByVal pMessage As String, ByVal pError As Boolean)
    ' Write to the application log
    Dim objLog As New EventLog("Application")

    objLog.Source = QUILOGY_APPLOG_SOURCE

    Try
      ' Make sure the source is registered, it should be
      If (Not EventLog.SourceExists(objLog.Source)) Then
        EventLog.CreateEventSource(objLog.Source, "Application")
      End If

      ' Write the message to the log
      If pError Then
        objLog.WriteEntry(pMessage, EventLogEntryType.Error)
      Else
        objLog.WriteEntry(pMessage, EventLogEntryType.Information)
      End If
    Catch e As Exception
      ' Not so important so skip it            
    Finally
      objLog.Dispose()
    End Try


  End Sub

  Public Sub LogText(ByVal pMessage As String, ByVal pFile As String, ByVal pError As Boolean, ByVal pFileOnly As Boolean)
    ' Write to both the text file and the Quilogy log

    Try
      ' Create or open the log file
      Dim objFs As FileStream = New FileStream(pFile, FileMode.OpenOrCreate, FileAccess.Write)
      Dim objSw As StreamWriter = New StreamWriter(objFs) ' create a stream writer 
      objSw.BaseStream.Seek(0, SeekOrigin.End) ' set the file pointer to the end

      ' Write the log entry        
      objSw.Write("Log Entry : ")
      objSw.Write("{0} {1}", DateTime.Now.ToShortTimeString(), DateTime.Now.ToShortDateString())
      objSw.WriteLine(":" & pMessage)
      objSw.Flush()  ' update underlying file
      objSw.Close()
    Catch e As Exception
      WriteLog("Could not create or open " & pFile & ": " & e.Message, True)
    Finally
      If Not pFileOnly Then
        Try
          If pError Then
            EventLog.WriteEntry(QUILOGY_CUSTLOG_SOURCE, pMessage, EventLogEntryType.Error)
          Else
            EventLog.WriteEntry(QUILOGY_CUSTLOG_SOURCE, pMessage, EventLogEntryType.Information)
          End If
        Catch e As Exception
          WriteLog("Could not write to Quilogy log.", True)
        End Try
      End If
    End Try

  End Sub

End Module

Public Class QXMLTransfer : Inherits ServiceBase

  ' This service will: write config information to the registry in the installer class
  ' watch for file changes to a directory (NTFS) or use the timers
  ' read the file and log something about it to a text log
  ' add something to the performance monitor counter and log it as well to the App log
  ' ftp the file to somewhere
  ' log success or failure

  Private mInterval As Integer
  Private mPath As String
  Private mFTP As String
  Private mProcess As Timer
  Private mLogFile As String
  Private mCriteria As String
  'Private mAppLog As EventLog
  Private mArchivePath As String
  Private mFtpReq As FtpWebRequest

  Public Sub New()

    'called only when first loaded not with each start and stop        
    MyBase.New()
    ' here the properties will be the same for all services, in main they can be changeable I guess?
    With Me
      .ServiceName = "QXMLTransfer"
      .CanShutdown = True
      .CanStop = True
      .CanPauseAndContinue = True
      .AutoLog = True
    End With

    ' Set the log file name
    mLogFile = "QXMLTransfer.log"
    mCriteria = "*.xml"
    mArchivePath = Environment.CurrentDirectory & "\Archive"

    ' Set up FTP
    mFtpReq = New FtpWebRequest(New Uri("ftp://" & mFTP))
    mFtpReq.Method = "PUT"

    ' called one time
    ' This call is required by the Component Designer.
    InitializeComponent()

  End Sub

  Private Function UploadFile(ByVal pSrcFile As String) As String
    Dim fsSrc As FileStream
    Dim dsDest As Stream
    Dim BytesRead As Integer = 0
    Dim ReadBuffer() As Byte
    Dim ftpResp As FtpWebResponse

    Const SIZETOREAD = 1024

    ' Open the stream for Reading
    Try
      fsSrc = New FileStream(pSrcFile, FileMode.Open, FileAccess.Read)
    Catch e As Exception
      Return e.Message & _
          " [Unable to open local file " + pSrcFile + " to upload]"
    End Try

    Try
      ' If openend successfully and not empty
      If fsSrc.Length > 0 Then
        dsDest = mFtpReq.GetRequestStream()
        Do ' Loop through the file and write it to the stream
          ReDim ReadBuffer(SIZETOREAD)
          BytesRead = fsSrc.Read(ReadBuffer, 0, SIZETOREAD)
          dsDest.Write(ReadBuffer, 0, BytesRead)
        Loop Until BytesRead = 0
      End If
      fsSrc.Close()
      dsDest.Close()

      ' Get the response
      ftpResp = mFtpReq.GetResponse()
      Return ftpResp.StatusDescription
    Catch e As Exception
      Return e.Message & _
          " [Unable to upload file " + pSrcFile & "]"
    End Try
  End Function


  Private Sub ProcessFiles()

    Dim objFile As FileInfo
    Dim strFile As String
    Dim intCount As Integer
    Dim objCounter As PerformanceCounter
    Dim objDir As Directory
    Dim ftpMyClient As FtpClient

    ' Set up the FTP call
    ftpMyClient = New FtpClient(mFTP)
    '    ftpMyClient.UserName = muser
    '    ftpMyClient.Password = mPwd

    Try

      'Go get each file
      For Each strFile In objDir.GetFileSystemEntries(mPath, mCriteria)
        objFile = New FileInfo(strFile)
        Try
          ' FTP it out                
          ftpMyClient.UploadFile(objFile.FullName, objFile.Name)
          ' Archive it
          objFile.MoveTo(mArchivePath & "\" & objFile.Name)
        Catch e As Exception
          ' Log the fact that an error occurred
          LogText("An error occurred processing file: " & e.Message, mLogFile, True, False)
        Finally
          LogText("Processed " & objFile.FullName, mLogFile, False, True)
          intCount += 1
        End Try
      Next
    Catch e As Exception

      ' Log the fact that an error occurred
      LogText("An error occurred processing files: " & e.Message, mLogFile, True, False)
    Finally

      ' Clean up the FTP            
      ftpMyClient = Nothing

      ' Write the number processed to the log
      LogText("Processed " & intCount & " files.", mLogFile, False, False)

      ' Increment Perf Counter
      If PerformanceCounterCategory.Exists("Quilogy XML Transfer") Then
        objCounter = New PerformanceCounter("Quilogy XML Transfer", "Files", False)
        objCounter.RawValue = intCount

        objCounter = New PerformanceCounter("Quilogy XML Transfer", "Total Files", False)
        objCounter.IncrementBy(intCount)
        objCounter.Dispose()
      Else
        LogText("Could not write performance information.", mLogFile, True, False)
      End If

    End Try

  End Sub

  Private Function GetRegValues() As Boolean
    Dim objKey As RegistryKey

    Try
      objKey = Registry.LocalMachine.OpenSubKey("System\CurrentControlSet\Services\QXMLTransfer\RunningValues", False)
      mInterval = CInt(objKey.GetValue("Timer", 30000))
      mPath = objKey.GetValue("Path").ToString
      mFTP = objKey.GetValue("FTPTo").ToString
    Catch e As Exception
      LogText("Could not get registry parameters.", mLogFile, True, False)
      Return False
    End Try

    Return True

  End Function
#Region " Component Designer generated code "

  'Required by the Component Designer
  Private components As System.ComponentModel.Container

  ' NOTE: The following procedure is required by the Component Designer
  ' It can be modified using the Component Designer.  Do not modify it
  ' using the code editor.
  Private Sub InitializeComponent()
    components = New System.ComponentModel.Container()
    Me.ServiceName = "QXMLTransfer"
  End Sub

#End Region

  Protected Overrides Sub OnStart(ByVal args() As String)
    ' TODO: Add code here to start your service. This method should set things
    ' in motion so your service can do its work.

    ' Read from the registry
    If Not GetRegValues() Then
      Throw New Exception("The service could not be started. See the event log for details.")
      Return
    End If

    ' Start the timer
    mProcess = New Timer(mInterval)
    AddHandler mProcess.Elapsed, AddressOf OnTimer
    mProcess.Start()

    ' Connect to the app log
    'mAppLog = New EventLog("Application")

    '       AddHandler mAppLog.EntryWritten, AddressOf NewAppLogEvent
    '      mAppLog.StartMonitoring()

    ' Log the fact that it started in the text log
    LogText("QXMLTransfer Service Started.", mLogFile, False, False)


  End Sub

  'Private Sub NewAppLogEvent(ByVal sender As Object, ByVal e As EventLogEvent)
  ' Check the Source
  '   If e.Entry.Source = "MSSQLServer" Then
  ' Look for specific messages
  '      If Instr(1, "terminating", e.Entry.Message, CompareMethod.Text) > 0 Then
  ' Stopping
  '         Me.OnStop()
  '        Return
  '  End If
  '   If instr(1, "pause", e.Entry.Message, CompareMethod.Text) > 0 Then
  ' Pausing
  '     Me.OnPause()
  '    Return
  'End If
  'End If
  'End Sub

  Private Sub OnTimer(ByVal source As Object, ByVal e As ElapsedEventArgs)
    ' Timer has gone off

    mProcess.Stop()
    LogText("QXMLTransfer Waking up", mLogFile, False, False)

    ' Now process the rest
    ProcessFiles()
    mProcess.Start()

  End Sub

  Private Sub TearDown()
    Dim objCounter As PerformanceCounter

    mProcess.Stop()
    objCounter = New PerformanceCounter("Quilogy XML Transfer", "Total Files", False)
    objCounter.RawValue = 0
    objCounter = New PerformanceCounter("Quilogy XML Transfer", "Files", False)
    objCounter.RawValue = 0

    objCounter.Dispose()

    mProcess.Close()
    mProcess.Dispose()
    'mAppLog.StopMonitoring()

    LogText("QXMLTransfer Service stopped.", mLogFile, False, False)

  End Sub
  Protected Overrides Sub OnStop()
    ' TODO: Add code here to perform any tear-down necessary to stop your service.
    TearDown()
  End Sub

  Protected Overrides Sub OnShutdown()
    ' TODO: Add code here to perform any tear-down necessary to stop your service.
    TearDown()
  End Sub

  Protected Overrides Sub OnPause()
    ' TODO: Add code here to perform any tear-down necessary to stop your service.
    mProcess.Stop()
    LogText("QXMLTransfer Service Paused.", mLogFile, False, False)
  End Sub

  Protected Overrides Sub OnContinue()
    ' TODO: Add code here to perform any tear-down necessary to stop your service.
    mProcess.Start()
    LogText("QXMLTransfer Service Continued.", mLogFile, False, False)
  End Sub

End Class




'******************* Install Code ***********************

<RunInstaller(True)> Public Class QXMLInstall : Inherits Installer

  ' For each service
  'Private mServiceInstaller As ServiceInstaller
  Private WithEvents mServiceInstaller As ServiceInstaller
  ' For the entire executable
  Private mProcessInstaller As ServiceProcessInstaller

  ' Performance Counter
  Private mPerfCounterInstaller As PerformanceCounterInstaller

  ' Event Log
  Private mELogInstaller As EventLogInstaller

  ' MSMQ
  Private mMSMQInstaller As MessageQueueInstaller


  Public Overrides Sub Install(ByVal state As IDictionary)
    ' Write default values to the registry
    Dim objKey As RegistryKey
    Dim objRun As RegistryKey

    MyBase.Install(state)
    Try
      objKey = Registry.LocalMachine.OpenSubKey("System\CurrentControlSet\Services\QXMLTransfer", True)
      objRun = objKey.CreateSubKey("RunningValues")
      objRun.SetValue("Timer", 15000)
      objRun.SetValue("Path", "c:\")
      objRun.SetValue("FTPTo", "ssosa")
    Catch e As Exception
      WriteLog("Default parameters were not created.", False)
    End Try

  End Sub

  Public Overrides Sub UnInstall(ByVal state As IDictionary)
    ' Write default values to the registry
    Dim objKey As RegistryKey

    Try
      objKey = Registry.LocalMachine.OpenSubKey("System\CurrentControlSet\Services\QXMLTransfer", True)
      objKey.DeleteSubKey("RunningValues")
    Catch e As Exception
      WriteLog("Default parameters were not uninstalled.", False)
    Finally
      MyBase.Uninstall(state)
    End Try
  End Sub

  Public Sub New()
    MyBase.New()

    ' Instantiate installers for process and services.
    mProcessInstaller = New ServiceProcessInstaller()
    mServiceInstaller() = New ServiceInstaller()
    mPerfCounterInstaller = New PerformanceCounterInstaller()
    mELogInstaller = New EventLogInstaller()
    mMSMQInstaller = New MessageQueueInstaller()


    ' Set up the perf counter
    Dim objCounter As New CounterCreationData("Total Files", "The total number of files processed by this service since it was restarted.", PerformanceCounterType.NumberOfItems32)
    Dim objCounter1 As New CounterCreationData("Files", "The number of files processed in the last interval.", PerformanceCounterType.NumberOfItems32)

    mPerfCounterInstaller.CategoryName = "Quilogy XML Transfer"
    mPerfCounterInstaller.Counters.Add(objCounter1)
    mPerfCounterInstaller.Counters.Add(objCounter)

    ' Set up the event log
    mELogInstaller.Log = "Quilogy Applications"
    mELogInstaller.Source = QUILOGY_CUSTLOG_SOURCE

    ' The services will run under the system account.
    mProcessInstaller.Account = ServiceAccount.LocalSystem

    ' The services will be started manually.
    mServiceInstaller().StartType = ServiceStartMode.Manual

    ' ServiceName must equal those on ServiceBase derived classes.            
    mServiceInstaller().ServiceName = "QXMLTransfer"
    mServiceInstaller().DisplayName = "Quilogy XML Transfer"

    With mMSMQInstaller
      .Label = "QuilgoyDocQueue"
      .UninstallAction = UninstallAction.Remove
      .UseJournalQueue = True
      .Transactional = True
      .Path = ".\Private$\QuilogyDocs"
    End With




    ' Add installers to collection. Order is not important.
    ' here is where you add performance monitor installers or event log installers
    Installers().Add(mServiceInstaller())
    Installers().Add(mProcessInstaller)
    Installers().Add(mPerfCounterInstaller)
    Installers().Add(mELogInstaller)
    'Installers().Add(mMSMQInstaller)
  End Sub

  ' Catch the events
  Public Sub mServiceInstaller_AfterInstall(ByVal sender As Object, ByVal e As System.Configuration.Install.InstallEventArgs) Handles mServiceInstaller.AfterInstall
    WriteLog("The service was installed successfully.", False)
  End Sub
  Public Sub mServiceInstaller_AfterUninstall(ByVal sender As Object, ByVal e As System.Configuration.Install.InstallEventArgs) Handles mServiceInstaller.AfterUninstall
    WriteLog("The service was uninstalled successfully.", False)
  End Sub

End Class




'******************* FTP Client ***********************
' Normally this code would be placed in its own assembly
' and reused in projects such as this service

Public Class FtpClient
  ' Simple FtpClient

  Public LastMessage As String
  Public UserName As String
  Public Password As String

  Private ftpUri As Uri

  Public Sub New(ByVal ftpSite As Uri)
    ftpUri = ftpSite
    InitClass()
  End Sub

  Public Sub New(ByVal ftpSite As String)
    ftpUri = New Uri(ftpSite)
    InitClass()
  End Sub

  Private Sub InitClass()
    ' Register the prefix
    Dim creator As FtpRequestCreator = New FtpRequestCreator()
    WebRequest.RegisterPrefix("ftp:", creator)
  End Sub

  Public Function UploadFile(ByVal source As String, ByVal dest As String) As Boolean
    Dim ftpReq As FtpWebRequest
    Dim ftpResp As FtpWebResponse

    ' Setup the request
    ftpReq = CType(WebRequest.Create(ftpUri), FtpWebRequest)
    ftpReq.Method = "PUT"

    ' Set the destination file
    ftpReq.m_szCmdParameter = dest

    ' Login if necessary
    If UserName.Length > 0 Then
      ftpReq.Credentials = New NetworkCredential(UserName, Password)
    End If

    Try
      LoadStream(source, ftpReq.GetRequestStream)
      ftpResp = CType(ftpReq.GetResponse, FtpWebResponse)

      ' See if it worked
      LastMessage = ftpResp.StatusDescription
      If ftpResp.Status = 226 Then
        Return True
      Else
        Throw New Exception("File Upload Failed [" _
          & source & "]:" & LastMessage)
        Return False
      End If

    Catch e As Exception
      LastMessage = e.Message
      Throw New Exception("File Upload Failed [" _
        & source & "]:" & LastMessage)
      Return False

    Finally
      ftpReq.CloseDataConnection()
      ftpReq = Nothing
      ftpResp = Nothing
    End Try

  End Function

  Private Function LoadStream(ByVal pSrcFile As String, ByRef s As Stream) As String
    Dim fsSrc As FileStream
    Dim BytesRead As Integer = 0
    Dim ReadBuffer() As Byte
    Dim ftpResp As FtpWebResponse

    Const SIZETOREAD As Integer = 1024

    ' Open the stream for Reading
    Try
      fsSrc = New FileStream(pSrcFile, FileMode.Open, FileAccess.Read)
    Catch e As Exception
      Throw New Exception(e.Message & _
          " [Unable to open local file " + pSrcFile + " to upload]")
    End Try

    Try
      ' If openend successfully and not empty
      If fsSrc.Length > 0 Then
        Do ' Loop through the file and write it to the stream
          ReDim ReadBuffer(SIZETOREAD)
          BytesRead = fsSrc.Read(ReadBuffer, 0, SIZETOREAD)
          s.Write(ReadBuffer, 0, BytesRead)
        Loop Until BytesRead = 0
      End If
      fsSrc.Close()
      s.Close()

    Catch e As Exception
      Throw New Exception(e.Message & _
          " [Unable to upload file " + pSrcFile & "]")
    End Try
  End Function

End Class