Option Explicit On 
Option Strict On

Imports System.Runtime.InteropServices
Imports System.EnterpriseServices
Imports System.Diagnostics
Imports System.Data.SqlClient

Namespace Quilogy.Education

  <ClassInterface(ClassInterfaceType.AutoDual), _
    Transaction(TransactionOption.Supported), _
    EventTrackingEnabled(True), _
    ConstructionEnabled(Enabled:=True, _
        Default:="server=ssosa;database=enrollment;trusted_connection=yes")> _
    Public MustInherit Class QuilogyDataAccessBase
    Inherits ServicedComponent

    Protected connect As String
    Protected cnSql As SqlConnection
    Public Debug As Boolean = False

    Public Sub New()

    End Sub
    Public Sub New(ByVal x As Integer)

    End Sub

    '*******************************************************************
    Public ReadOnly Property ConstructString() As String
      Get
        Return connect
      End Get
    End Property

    '*******************************************************************
    Protected NotOverridable Overrides Sub Construct(ByVal s As String)
      ' Implements object construction
      connect = s
      cnSql = New SqlConnection(connect)
    End Sub

    '*******************************************************************
    Protected Overrides Function CanBePooled() As Boolean
      ' Default is that the objects will not be pooled
      Return False
    End Function

    '*******************************************************************
    Protected Overrides Sub Activate()
      If Debug Then
        WriteTrace("Activate")
      End If
    End Sub

    '*******************************************************************
    Protected Overrides Sub Deactivate()
      If Debug Then
        WriteTrace("Deactivate")
      End If
    End Sub

    '*******************************************************************
    Public Sub WriteTrace(ByVal eventMessage As String)
      ' Writes the event to the Application log
      Dim objLog As EventLog

      Try
        objLog = New EventLog("Application")
        objLog.Source = Me.GetType.ToString

        If (Not EventLog.SourceExists(objLog.Source)) Then
          EventLog.CreateEventSource(objLog.Source, _
            "Application")
        End If

        ' Write the message to the log
        objLog.WriteEntry(DebugTrace(eventMessage), _
          EventLogEntryType.Information)
        objLog.Dispose()

      Catch e As Exception
        ' Not important so skip it
      Finally
        ' Make sure the transaction if any can commit
        If ContextUtil.IsInTransaction Then
          ContextUtil.EnableCommit()
        End If
      End Try
    End Sub

    '*******************************************************************
    Protected Function DebugTrace(ByVal eventMessage As String) As String
      ' Used to write events to the application event log
      Dim strTrace As String

      strTrace = "Class: " & Me.GetType.ToString & vbCrLf
      strTrace &= "Event: " & eventMessage & vbCrLf
      strTrace &= "Construct String: " & connect & vbCrLf
      strTrace &= "Activity:" & ContextUtil.ActivityId.ToString & vbCrLf
      strTrace &= "Context:" & ContextUtil.ContextId.ToString & vbCrLf
      strTrace &= "Transaction?" & ContextUtil.IsInTransaction.ToString & vbCrLf
      strTrace &= "Security?" & ContextUtil.IsSecurityEnabled.ToString & vbCrLf
      If ContextUtil.IsInTransaction Then
        strTrace &= "TransactionId:" & ContextUtil.TransactionId.ToString & vbCrLf
        strTrace &= "Direct Caller:" & SecurityCallContext.CurrentCall.DirectCaller.AccountName & vbCrLf
        strTrace &= "Original Caller:" & SecurityCallContext.CurrentCall.OriginalCaller.AccountName
      End If

      Return strTrace
    End Function




  End Class


  '*******************************************************************
  ' Event interface
  Public Interface IOfferingEvents
    Sub NewClass(ByVal classID As Integer, ByVal location As Integer)
    Sub ClassCancelled(ByVal classID As Integer, ByVal location As Integer)
  End Interface

  '*******************************************************************
  ' Event class
  <EventClassAttribute()> _
  Public Class OfferingsEvents : Inherits ServicedComponent
    Implements IOfferingEvents

    Public Sub NewClass(ByVal classID As Integer, _
      ByVal location As Integer) Implements IOfferingEvents.NewClass
      ' do something
    End Sub

    Public Sub ClassCancelled(ByVal classID As Integer, _
      ByVal location As Integer) Implements IOfferingEvents.ClassCancelled
      ' do something
    End Sub
  End Class

  '*******************************************************************
  '    Public Interface IQuilogyQuery
  ' Interfaced used to query 
  ' Function GetByID(ByVal pID As Long) As DataSet
  '       Function GetByName(ByVal pName As String) As DataSet
  '   End Interface

End Namespace

