Option Explicit On 
Option Strict Off

Imports System
Imports System.Data
Imports System.Data.SqlClient
Imports Quilogy.Education
Imports System.EnterpriseServices
Imports System.Runtime.InteropServices

Namespace Quilogy.Education.Schedule

    Public Enum RegistrationType
        Web = 1
        Phone = 2
    End Enum

  <ClassInterface(ClassInterfaceType.AutoDual)> _
  Public Class Classes ': Inherits Quilogy.Education.QuilogyDataAccess

    'this should be com+ class
    Private cnSQL As SqlConnection ' Take out when Serviced
    Private strConnect As String  ' Take out when Serviced

    '*******************************************************************
    Public Sub New()
      MyBase.New()
    End Sub

    '*******************************************************************
    Public Sub New(ByVal debug As Boolean)
      MyBase.New()
      'Me.Debug = debug
    End Sub

    '*******************************************************************
    Public Sub New(ByVal connect As String)
      ' Overloaded constructor so that the class can be used without object construction
      MyBase.New()
      strConnect = connect
      cnSQL = New SqlConnection(strConnect)
    End Sub

    '*******************************************************************
    Public Sub New(ByVal connect As String, ByVal debug As Boolean)
      ' Overloaded constructor so that the class can be used without object construction
      MyBase.New()
      strConnect = connect
      cnSQL = New SqlConnection(strConnect)

      '      Me.Debug = pDebug
    End Sub

    '*******************************************************************
    '<ComVisible(False), AutoComplete(True)> _
    Public Function Save(ByVal courseData As DataSet) As Integer
      Dim intClassID, intLocation As Integer
      Dim objOffEvent As OfferingsEvents

      ' Save the class here

      ' Fire the event
      objOffEvent = New OfferingsEvents()
      objOffEvent.NewClass(intClassID, intLocation)
    End Function

    '<AutoComplete(True)> _
    Public Function IsFull(ByVal classID As Integer) As Boolean

      ' Queries the database to determine if the class is full

      Dim cmSql As SqlCommand
      Dim parmSql As SqlParameter

      cmSql = New SqlCommand("usp_IsFull", cnSQL)

      ' Create the parms
      With cmSql
        .CommandType = CommandType.StoredProcedure
        .Parameters.Add(New SqlParameter("@ClassID", _
          SqlDbType.Int))
        .Parameters("@ClassID").Value = classID
      End With

      ' Set up the return value
      parmSql = New SqlParameter("RETURN", SqlDbType.Int)
      parmSql.Direction = ParameterDirection.ReturnValue
      cmSql.Parameters.Add(parmSql)

      ' Call the proc
      Try
        cnSQL.Open()
        cmSql.ExecuteNonQuery()
        cnSQL.Close()
      Catch e As Exception
        Throw New Exception(cnSQL.ConnectionString & e.Message)
        Return False
      End Try

      ' Is the class full?
      If CType(parmSql.Value, Integer) = 1 Then
        Return True
      Else
        Return False
      End If

    End Function

    '<AutoComplete(True)> _
    Public Function CalcCost(ByVal classID As Integer, ByVal regType As RegistrationType) As Double

      Dim decDiscount As Double
      Dim decCost As Double
      Dim cmSql As SqlCommand
      Dim parmSql As SqlParameter

      ' Calculate the discount
      If regType = RegistrationType.Web Then
        decDiscount = 0.1
      Else
        decDiscount = 0
      End If

      ' Get the cost
      cmSql = New SqlCommand("usp_CalcCost", cnSQL)

      ' Create the parms
      With cmSql
        .CommandType = CommandType.StoredProcedure
        .Parameters.Add(New SqlParameter("@ClassID", SqlDbType.Int))
        .Parameters("@ClassID").Value = classID
      End With

      ' Set up the return value
      parmSql = New SqlParameter("RETURN", SqlDbType.Int)
      parmSql.Direction = ParameterDirection.ReturnValue
      cmSql.Parameters.Add(parmSql)

      ' Call the proc
      cnSQL.Open()
      cmSql.ExecuteNonQuery()
      cnSQL.Close()

      ' Calculate the Cost
      decCost = CType(parmSql.Value, Double)
      decDiscount = decDiscount * decCost
      Return decCost - decDiscount
    End Function

    '<ComVisible(False), AutoComplete(True)> _
    Public Function AddEnrollment(ByVal enroll As EnrollmentData) As Integer
      ' Adds the enrollment to the database

      Dim cmSql As SqlCommand
      Dim parmSql As SqlParameter
      Dim intClassID As Integer
      Dim intStudentID As Integer
      Dim flWebEnroll As Boolean = False
      Dim decCost As Decimal
      Dim rEnroll As DataRow
      Dim arStud() As DataRow

      ' Check the incoming parameter
      If enroll Is Nothing Then
        Throw New ArgumentNullException("No data found!")
        Return 0
      End If

      ' Parse the dataset
      rEnroll = enroll.EnrollmentTable.Rows(0)
      intClassID = rEnroll(EnrollmentData.CLASS_ID)
      If rEnroll(EnrollmentData.WEB_ENROLL) = "Y" Then
        flWebEnroll = True
      End If
      decCost = rEnroll(EnrollmentData.COST)
      arStud = rEnroll.GetChildRows("Enrollment_Student")
      intStudentID = arStud(0).Item(EnrollmentData.STUDENT_ID)


      ' Add the enrollment
      cmSql = New SqlCommand("usp_AddEnrollment", cnSQL)

      ' Create the parms
      With cmSql
        .CommandType = CommandType.StoredProcedure
        .Parameters.Add(New SqlParameter("@Cost", SqlDbType.Money))
        .Parameters("@Cost").Value = decCost
        .Parameters.Add(New SqlParameter("@ClassID", SqlDbType.Int))
        .Parameters("@ClassID").Value = intClassID
        .Parameters.Add(New SqlParameter("@StudentID", SqlDbType.Int))
        .Parameters("@StudentID").Value = intStudentID
        .Parameters.Add(New SqlParameter("@WebEnroll", SqlDbType.Char))
        .Parameters("@WebEnroll").Value = flWebEnroll
      End With

      ' Set up the return value
      parmSql = New SqlParameter("RETURN", SqlDbType.Int)
      parmSql.Direction = ParameterDirection.ReturnValue
      cmSql.Parameters.Add(parmSql)

      ' Call the proc
      cnSQL.Open()
      cmSql.ExecuteNonQuery()
      cnSQL.Close()

      ' Return the enrollment ID
      Return CType(parmSql.Value, Integer)
    End Function

    '<AutoComplete(True)> _
    Public Function GetSchedule(ByVal locID As Integer, _
      ByVal vendorID As Integer, _
      Optional ByVal daysOut As Byte = 30) As ScheduleData
      ' Returns the course schedule

      Dim dsSched As New ScheduleData()
      Dim daSql As SqlDataAdapter
      Dim parmSql As SqlParameter
      Dim drC As DataRow
      Dim strLocation As String
      Dim strVendor As String

      ' Setup the data adapter
      daSql = New SqlDataAdapter("usp_GetSchedule", cnSQL)

      ' Call the stored procedure
      daSql.SelectCommand.CommandType = CommandType.StoredProcedure
      parmSql = daSql.SelectCommand.Parameters.Add(New SqlParameter("@LocID", SqlDbType.Int))
      parmSql.Value = locID
      parmSql = daSql.SelectCommand.Parameters.Add(New SqlParameter("@VendorID", SqlDbType.Int))
      parmSql.Value = vendorID
      parmSql = daSql.SelectCommand.Parameters.Add(New SqlParameter("@Advance", SqlDbType.Int))
      parmSql.Value = daysOut

      daSql.MissingSchemaAction = MissingSchemaAction.Ignore
      daSql.Fill(dsSched, dsSched.OfferingsTable.TableName)

      ' Now fill in the criteria
      drC = dsSched.CriteriaTable.NewRow()
      drC(dsSched.QUERY_LOCATION) = strLocation
      drC(dsSched.QUERY_VENDOR) = strVendor
      drC(dsSched.ADVANCE) = daysOut

      Return dsSched
    End Function
  End Class

End Namespace