Option Explicit On 
Option Strict On

Imports System.Data
Imports System.Data.SqlClient
Imports Quilogy.Education
Imports System.EnterpriseServices
Imports System.Runtime.InteropServices

Namespace Quilogy.Education.Enrollment

    <ClassInterface(ClassInterfaceType.AutoDual)> _
    Public Class Students ': Inherits QuilogyDataAccess
        ' Students Data Access Class


        Private cnSQL As SqlConnection ' Take out when Serviced
        Private strConnect As String  ' Take out when Serviced

        '*******************************************************************
        Public Sub New()
            MyBase.new()
        End Sub

        '*******************************************************************
        Public Sub New(ByVal debug As Boolean)
            MyBase.New()
            '    Me.Debug = pDebug
        End Sub

        '*******************************************************************
        Public Sub New(ByVal connect As String)
            ' Overloaded constructor so that the class can be used without object construction
            MyBase.New()
            strConnect = connect
            cnSQL = New SqlConnection(strConnect)
        End Sub

        '*******************************************************************
        Public Sub New(ByVal connect As String, ByVal debug As Boolean)
            ' Overloaded constructor so that the class can be used without object construction
            MyBase.New()
            strConnect = Connect
            cnSQL = New SqlConnection(strConnect)

            '    Me.Debug = pDebug
        End Sub

        '*************************************************
        '<AutoComplete(True)> _
        Public Function GetTranscript(ByVal studentID As Integer) As TranscriptData
            ' Fill a dataset with transcript data based on the student ID

            Dim dsTs As New TranscriptData()
            Dim daSql As SqlDataAdapter
            Dim parmSql As SqlParameter

            ' Check the incoming paramters
            If studentID <= 0 Then
                Throw New ArgumentOutOfRangeException("StudentID cannot be less than or equal to 0")
                Return Nothing
            End If

            ' Setup the data adapter
            daSql = New SqlDataAdapter("usp_GetTranscript", cnSQL)

            ' Call the stored procedure
            daSql.SelectCommand.CommandType = CommandType.StoredProcedure
            parmSql = daSql.SelectCommand.Parameters.Add(New SqlParameter("@StudentID", SqlDbType.Int))
            parmSql.Value = studentID

            daSql.MissingSchemaAction = MissingSchemaAction.Ignore
            daSql.Fill(dsTs, dsTs.ClassTable.TableName)
            Return dsTs

        End Function


        '*************************************************
        '<AutoComplete(True)> _
        Public Function Login(ByVal email As String, ByVal pwd As String) As StudentData
            ' Login the user and return the demographic data

            Dim dsStud As New StudentData()
            Dim daSql As SqlDataAdapter
            Dim parmSql As SqlParameter

            ' Check the incoming paramters
            If email.Length = 0 Or pwd.Length = 0 Then
                Throw New ArgumentNullException("Must supply email address and password")
                Return Nothing
            End If

            ' Setup the data adapter
            daSql = New SqlDataAdapter("usp_LoginStudent", cnSQL)

            ' Call the stored procedure
            daSql.SelectCommand.CommandType = CommandType.StoredProcedure
            parmSql = daSql.SelectCommand.Parameters.Add(New SqlParameter("@Email", SqlDbType.VarChar))
            parmSql.Value = email
            parmSql = daSql.SelectCommand.Parameters.Add(New SqlParameter("@Pwd", SqlDbType.VarChar))
            parmSql.Value = pwd

            daSql.MissingSchemaAction = MissingSchemaAction.Ignore
            daSql.Fill(dsStud, dsStud.StudentTable.TableName)
            Return dsStud

        End Function

        '*************************************************
    '<AutoComplete(True)> _
    Public Function ChangePass(ByVal email As String, ByVal oldPwd As String, ByVal newPwd As String) As Boolean
      ' execute a proc
    End Function

    '*************************************************
    '<ComVisible(False), AutoComplete(True)> _
    Public Function GetStudent(ByVal studentID As Integer) As StudentData

      Dim dsStud As New StudentData()
      Dim daSql As SqlDataAdapter
      Dim parmSql As SqlParameter

      ' Check the incoming paramters
      If studentID <= 0 Then
        Throw New ArgumentOutOfRangeException("StudentID cannot be less than or equal to 0")
        Return Nothing
      End If

      ' Setup the data adapter
      daSql = New SqlDataAdapter("usp_GetStudent", cnSQL)

      ' Call the stored procedure
      daSql.SelectCommand.CommandType = CommandType.StoredProcedure
      parmSql = daSql.SelectCommand.Parameters.Add(New SqlParameter("@StudentID", SqlDbType.Int))
      parmSql.Value = studentID

      daSql.MissingSchemaAction = MissingSchemaAction.Ignore
      daSql.Fill(dsStud, dsStud.StudentTable.TableName)
      Return dsStud

    End Function

    '*************************************************
    '<ComVisible(False), AutoComplete(True)> _
    Public Function Save(ByVal student As StudentData) As Integer
      ' Handles both inserts and updates through the proc and returns the new StudentID

      Dim cmSql As SqlCommand
      Dim parmSql As SqlParameter

      ' Check the incoming parameter
      If student Is Nothing Then
        Throw New ArgumentNullException("No data found!")
        Return 0
      End If

      cmSql = New SqlCommand("usp_SaveStudent", cnSQL)

      ' Create the parms and extract the data
      With cmSql
        .CommandType = CommandType.StoredProcedure
        .Parameters.Add(New SqlParameter("@StudentID", SqlDbType.Int))
        .Parameters("@StudentID").Value = student.StudentTable.Rows(0).Item(StudentData.STUDENT_ID)
        .Parameters.Add(New SqlParameter("@FName", SqlDbType.VarChar))
        .Parameters("@FName").Value = student.StudentTable.Rows(0).Item(StudentData.FIRST_NAME)
        .Parameters.Add(New SqlParameter("@LName", SqlDbType.VarChar))
        .Parameters("@LName").Value = student.StudentTable.Rows(0).Item(StudentData.LAST_NAME)
        .Parameters.Add(New SqlParameter("@Company", SqlDbType.VarChar))
        .Parameters("@Company").Value = student.StudentTable.Rows(0).Item(StudentData.ORGANIZATION)
        .Parameters.Add(New SqlParameter("@Email", SqlDbType.VarChar))
        .Parameters("@Email").Value = student.StudentTable.Rows(0).Item(StudentData.EMAIL)
        .Parameters.Add(New SqlParameter("@Address", SqlDbType.VarChar))
        .Parameters("@Address").Value = student.StudentTable.Rows(0).Item(StudentData.ADDRESS)
        .Parameters.Add(New SqlParameter("@City", SqlDbType.VarChar))
        .Parameters("@City").Value = student.StudentTable.Rows(0).Item(StudentData.CITY)
        .Parameters.Add(New SqlParameter("@State", SqlDbType.VarChar))
        .Parameters("@State").Value = student.StudentTable.Rows(0).Item(StudentData.STATE)
        .Parameters.Add(New SqlParameter("@ZipCode", SqlDbType.VarChar))
        .Parameters("@ZipCode").Value = student.StudentTable.Rows(0).Item(StudentData.ZIP_CODE)
        .Parameters.Add(New SqlParameter("@Phone", SqlDbType.VarChar))
        .Parameters("@Phone").Value = student.StudentTable.Rows(0).Item(StudentData.PHONE)
      End With

      ' Set up the return value
      parmSql = New SqlParameter("RETURN", SqlDbType.Int)
      parmSql.Direction = ParameterDirection.ReturnValue
      cmSql.Parameters.Add(parmSql)

      ' Call the proc
      cnSQL.Open()
      cmSql.ExecuteNonQuery()
      cnSQL.Close()

      ' Return the new or old student id
      Return CType(parmSql.Value, Integer)

    End Function

    '*************************************************
    '<AutoComplete(True)> _
    '        Public Function Remove(ByVal studentID() As Integer) As Boolean
    ' Add one or all??
    '       End Function

    '*************************************************
    '<AutoComplete(True)> _
    Public Function CheckIfExists(ByVal email As String, ByVal lastName As String) As Integer

      Dim cmSql As SqlCommand
      Dim parmSql As SqlParameter

      ' Check the incoming parameter
      If email.Length = 0 Or lastName.Length = 0 Then
        Throw New ArgumentNullException("Must supply email address and last name")
        Return 0
      End If

      cmSql = New SqlCommand("usp_StudentExists", cnSQL)

      ' Create the parms and extract the data
      With cmSql
        .CommandType = CommandType.StoredProcedure
        .Parameters.Add(New SqlParameter("@LName", SqlDbType.VarChar))
        .Parameters("@LName").Value = lastName
        .Parameters.Add(New SqlParameter("@Email", SqlDbType.VarChar))
        .Parameters("@Email").Value = email
        .Parameters.Add(New SqlParameter("@StudentID", SqlDbType.Int))
        .Parameters("@StudentID").Direction = ParameterDirection.Output
      End With

      ' Call the proc
      cnSQL.Open()
      cmSql.ExecuteNonQuery()
      cnSQL.Close()

      ' Check if Null before Return
      If cmSql.Parameters("@StudentID").Value Is DBNull.Value Then
        Return 0
      Else
        Return CType(cmSql.Parameters("@StudentID").Value, Integer)
      End If

    End Function

    '*************************************************
    '<AutoComplete(True)> _
    Public Function ClassesTaken(ByVal studentID As Integer) As Integer
      ' Get the number of classes taken by this student

      Dim cmSql As SqlCommand
      Dim parmSql As SqlParameter

      ' Check the incoming parameter
      If studentID <= 0 Then
        Throw New ArgumentOutOfRangeException("StudentID must be greater than 0")
        Return 0
      End If

      cmSql = New SqlCommand("usp_GetClasses", cnSQL)

      ' Create the parms and extract the data
      With cmSql
        .CommandType = CommandType.StoredProcedure
        .Parameters.Add(New SqlParameter("@StudentID", SqlDbType.Int))
        .Parameters("@StudentID").Value = studentID
      End With

      ' Call the proc
      cnSQL.Open()
      ClassesTaken = CType(cmSql.ExecuteScalar, Integer)
      cnSQL.Close()

    End Function

  End Class
End Namespace

