Option Strict On
Option Explicit On 

Imports System
Imports System.Data
Imports System.Runtime.Serialization

Namespace Quilogy.Education
  <Serializable()> _
Public Class ScheduleData : Inherits DataSet

    Private offering As DataTable
    Private criteria As DataTable

    Public Const QUERY_VENDOR As String = "QVendor"
    Public Const QUERY_LOCATION As String = "QLocation"
    Public Const ADVANCE As String = "Advance"
    Public Const CLASS_ID As String = "ClassID"
    Public Const LOCATION As String = "City"
    Public Const LOCATION_ID As String = "LocID"
    Public Const DAYS As String = "Days"
    Public Const COURSE_NUM As String = "CourseNum"
    Public Const COURSE_ID As String = "CourseID"
    Public Const COURSE_DESC As String = "Description"
    Public Const SYLLABUS As String = "Syllabus"
    Public Const CLASS_DATE As String = "ClassDate"
    Public Const START_TIME As String = "StartTime"
    Public Const PRODUCT As String = "Product"
    Public Const PRODUCT_ID As String = "ProductID"
    Public Const VENDOR As String = "Vendor"
    Public Const VENDOR_ID As String = "VendorID"

    Public Sub New()
      '   Initialize a TranscriptData instance by building the table              
      MyBase.New()
      Me.InitClass()
    End Sub

    Private Sub InitClass()
      ' Create the tables in the dataset
      BuildTables()

      ' Initialize the class
      Me.DataSetName = "QuilogySchedule"
      Me.Namespace = "www.quilogy.com/education/schedule"
      Me.Prefix = "qeds"
      Me.CaseSensitive = False
    End Sub

    Private Sub New(ByVal info As SerializationInfo, ByVal context As StreamingContext)
      MyBase.New()
      Me.InitClass()
      Me.GetSerializationData(info, context)
    End Sub

    Public ReadOnly Property OfferingsTable() As DataTable
      Get
        Return offering
      End Get
    End Property

    Public ReadOnly Property CriteriaTable() As DataTable
      Get
        Return criteria
      End Get
    End Property

    Private Sub BuildTables()

      ' Create the table
      criteria = New DataTable("Criteria")

      With criteria.Columns
        .Add(QUERY_VENDOR, GetType(System.String))
        .Add(QUERY_LOCATION, GetType(System.String))
        .Add(ADVANCE, GetType(System.Int32))
      End With

      ' Set the column attributes
      With criteria
        .Columns(QUERY_LOCATION).Caption = "QueryLocation"
        .Columns(QUERY_VENDOR).Caption = "QueryVendor"
      End With


      ' Create the table
      offering = New DataTable("Offering")

      With offering.Columns
        .Add(CLASS_ID, GetType(System.Int32))
        .Add(LOCATION, GetType(System.String))
        .Add(LOCATION_ID, GetType(System.Int32))
        .Add(DAYS, GetType(System.Int32))
        .Add(COURSE_NUM, GetType(System.String))
        .Add(COURSE_ID, GetType(System.Int32))
        .Add(COURSE_DESC, GetType(System.String))
        .Add(SYLLABUS, GetType(System.String))
        .Add(CLASS_DATE, GetType(System.String))
        .Add(START_TIME, GetType(System.String))
        .Add(PRODUCT, GetType(System.String))
        .Add(PRODUCT_ID, GetType(System.Int32))
        .Add(VENDOR, GetType(System.String))
        .Add(VENDOR_ID, GetType(System.Int32))
      End With

      ' Set the column attributes
      With offering

        .Columns(CLASS_ID).ColumnMapping = MappingType.Attribute
        .Columns(LOCATION_ID).ColumnMapping = MappingType.Attribute
        .Columns(PRODUCT_ID).ColumnMapping = MappingType.Attribute
        .Columns(VENDOR_ID).ColumnMapping = MappingType.Attribute
        .Columns(COURSE_ID).ColumnMapping = MappingType.Attribute
      End With

      ' Add the table to the collection
      Me.Tables.Add(criteria)
      Me.Tables.Add(offering)

    End Sub
  End Class
End Namespace
