Option Strict On
Option Explicit On 

Imports System
Imports System.Data
Imports System.Runtime.Serialization

Namespace Quilogy.Education

  <Serializable()> _
  Public Class TranscriptData : Inherits DataSet

    Private studentClass As DataTable

    Public Event ValidationWarning(ByVal pMessage As String)

    Public Const STUDENT_ID As String = "StudentID"
    Public Const ENROLL_ID As String = "EnrollID"
    Public Const CLASS_DATE As String = "Date"
    Public Const CITY As String = "City"
    Public Const INSTRUCTOR As String = "Instructor"
    Public Const INS_EMAIL As String = "InstructorEmail"
    Public Const VENDOR_NAME As String = "VendorName"
    Public Const COURSE_NUM As String = "CourseNum"
    Public Const COURSE_DESC As String = "CourseDesc"
    Public Const DAYS As String = "Days"
    Public Const ORGANIZATION As String = "StudentOrganization"

    Public Sub New()
      '   Initialize a TranscriptData instance by building the table              
      MyBase.New()
      InitClass()
    End Sub

    Public Sub New(ByVal info As SerializationInfo, ByVal context As StreamingContext)
      '   Initialize a TranscriptData instance 
      MyBase.New()
      InitClass()
      Me.GetSerializationData(info, context)
    End Sub

    Private Sub InitClass()
      ' Create the tables in the dataset
      BuildTables()
      ' Initialize the class
      Me.DataSetName = "QuilogyTranscript"
      Me.Namespace = "www.quilogy.com/education/transcript"
      Me.Prefix = "qedts"
      Me.CaseSensitive = False

      ' Capture the ColumnChangingEvent
      AddHandler studentClass.ColumnChanging, AddressOf Me.TranscriptColChanging
    End Sub

    Private Sub TranscriptColChanging(ByVal sender As Object, ByVal e As DataColumnChangeEventArgs)
      Dim strOrig As String
      ' Do simple data validation
      Select Case e.Column.ColumnName
        Case COURSE_DESC
          ' Truncate long descriptions
          If Len(e.ProposedValue) > 255 Then
            strOrig = CType(e.ProposedValue, String)
            e.ProposedValue = Left(CType(e.ProposedValue, String), 255)
            RaiseEvent ValidationWarning("The " & COURSE_DESC & " '" & strOrig & _
     "' was truncated to '" & CType(e.ProposedValue, String) & "'")
          End If
      End Select
    End Sub

    Public ReadOnly Property ClassTable() As DataTable
      Get
        Return studentClass
      End Get
    End Property

    Private Sub BuildTables()
      Dim pk() As DataColumn

      ' Create the transcript table
      studentClass = New DataTable("Class")

      With studentClass.Columns
        .Add(STUDENT_ID, GetType(System.Int32))
        .Add(ENROLL_ID, GetType(System.Int32))
        .Add(CLASS_DATE, GetType(System.DateTime))
        .Add(CITY, GetType(System.String))
        .Add(INSTRUCTOR, GetType(System.String))
        .Add(INS_EMAIL, GetType(System.String))
        .Add(VENDOR_NAME, GetType(System.String))
        .Add(COURSE_NUM, GetType(System.String))
        .Add(COURSE_DESC, GetType(System.String))
        .Add(DAYS, GetType(System.Byte))
        .Add(ORGANIZATION, GetType(System.String))
      End With

      ' Set the column attributes
      With studentClass
        .Columns(STUDENT_ID).ColumnMapping = MappingType.Attribute
        .Columns(ENROLL_ID).ColumnMapping = MappingType.Hidden
        .Columns(CLASS_DATE).AllowDBNull = False
        .Columns(CLASS_DATE).Caption = "Class Date"
        .Columns(CITY).AllowDBNull = False
        .Columns(COURSE_NUM).AllowDBNull = False
        .Columns(DAYS).DefaultValue = 0
        .Columns(VENDOR_NAME).Caption = "Vendor Name"
        .Columns(COURSE_NUM).Caption = "Course Number"
        .Columns(COURSE_DESC).Caption = "Description"
        .Columns(ORGANIZATION).Caption = "Your Organization"
      End With

      ' Set the primary key
      ReDim pk(1)
      pk(0) = studentClass.Columns(ENROLL_ID)
      studentClass.PrimaryKey = pk

      ' Add the table to the collection
      Me.Tables.Add(studentClass)

    End Sub
  End Class
End Namespace
